<?php

namespace Drupal\cosign\CosignFunctions;

use Drupal\user\Entity\User;

/**
 * Cosign shared functions.
 */
class CosignSharedFunctions implements CosignSharedFunctionsInterface {

  /**
   * {@inheritdoc}
   */
  public function cosignUserStatus($cosign_username) {
    $user = \Drupal::currentUser();
    $uname = $user->getAccountName();
    $drupal_user = user_load_by_name($cosign_username);
    if ($drupal_user && $drupal_user->isBlocked()) {
      if (\Drupal::config('cosign.settings')->get('cosign_allow_anons_on_https') == 1) {
        return User::load(0);
      }
      else {
        return NULL;
      }
    }
    if (!empty($uname)) {
      // Youre already logged in
      // make sure you are the cosign user. if not log out. This is unlikely.
      if ($cosign_username != $uname) {
        user_logout();
        return NULL;
      }
    }
    if (!empty($cosign_username)) {
      $is_friend_account = $this->cosignIsFriendAccount($cosign_username);
      // If friend accounts are not allowed, log them out.
      if (\Drupal::config('cosign.settings')->get('cosign_allow_friend_accounts') == 0 && $is_friend_account) {
        $this->cosignFriendNotAllowed();
        if (\Drupal::config('cosign.settings')->get('cosign_allow_anons_on_https') == 1) {
          return User::load(0);
        }
        else {
          return NULL;
        }
      }
    }
    if (!empty($cosign_username) && !empty($drupal_user) && empty($uname)) {
      // Login the cosign user.
      $this->cosignLoginUser($drupal_user);
    }
    elseif (!empty($cosign_username) && empty($drupal_user)) {
      // Cosign user doesn't have a drupal account.
      if (\Drupal::config('cosign.settings')->get('cosign_autocreate') == 1) {
        $new_user = $this->cosignCreateNewUser($cosign_username);
        User::load($new_user->id());
      }
      else {
        User::load(0);
      }
    }
    elseif (empty($cosign_username) && \Drupal::config('cosign.settings')->get('cosign_allow_anons_on_https') == 0) {
      // No cosign account found.
      user_logout();
      return NULL;
    }
    $user = \Drupal::currentUser();
    if (!$user) {
      $user = User::load(0);
    }
    if ($user->id() == 0 && \Drupal::config('cosign.settings')->get('cosign_allow_anons_on_https') == 1) {
    }
    return $user;
  }

  /**
   * {@inheritdoc}
   */
  public function cosignLoginUser($drupal_user) {
    user_login_finalize($drupal_user);
    $the_user = \Drupal::currentUser();
    $username = $this->cosignUserMasqueradeCheck();

    if ($the_user->getAccountName() != $username) {
      \Drupal::logger('cosign')->notice('User attempted login and the cosign username: @remote_user, did not match the drupal username: @drupal_user', ['@remote_user' => $username, '@drupal_user' => $the_user->getAccountName()]);
      user_logout();
    }

    return User::load($the_user->id());
  }

  /**
   * {@inheritdoc}
   */
  public function cosignFriendNotAllowed() {
    \Drupal::logger('cosign')->notice('User attempted login using a university friend account and the friend account configuration setting is turned off: @remote_user', ['@remote_user' => '']);
    \Drupal::messenger()->addMessage(t('%message', ['%message' => \Drupal::config('cosign.settings')->get('cosign_friend_account_message')]), 'warning');
    if (\Drupal::config('cosign.settings')->get('cosign_allow_anons_on_https') == 1) {
      $cosign_brand = \Drupal::config('cosign.settings')->get('cosign_branded');
      \Drupal::messenger()->addMessage(t('You might want to <a href="/user/logout">logout of %cosignBrand</a> to browse anonymously or as another %cosignBrand user.', ['%cosignBrand' => $cosign_brand]), 'warning');
    }
    else {
      user_logout();
    }
  }

  /**
   * {@inheritdoc}
   */
  public function cosignLogoutUrl() {
    $logout_path = \Drupal::config('cosign.settings')->get('cosign_logout_path');
    $logout_to = \Drupal::config('cosign.settings')->get('cosign_logout_to') . '/';
    return $logout_path . '?' . $logout_to;
  }

  /**
   * {@inheritdoc}
   */
  public function cosignRetrieveRemoteUser() {
    $cosign_name = '';
    // Make sure we get the remote user whichever way it is available.
    if (isset($_SERVER['REDIRECT_REMOTE_USER'])) {
      $cosign_name = $_SERVER['REDIRECT_REMOTE_USER'];
    }
    elseif (isset($_SERVER['REMOTE_USER'])) {
      $cosign_name = $_SERVER['REMOTE_USER'];
    }

    return $cosign_name;
  }

  /**
   * {@inheritdoc}
   */
  public function cosignIsHttps() {
    $is_https = FALSE;
    if (\Drupal::request()->server->get('protossl') == 's') {
      $is_https = TRUE;
    }

    return $is_https;
  }

  /**
   * {@inheritdoc}
   */
  public function cosignIsFriendAccount($username) {
    // Make sure we get friend whichever way it is available.
    $is_friend_account = FALSE;
    if ((isset($_SERVER['REMOTE_REALM']) && $_SERVER['REMOTE_REALM'] == 'friend') || stristr($username, '@')) {
      $is_friend_account = TRUE;
    }

    return $is_friend_account;
  }

  /**
   * {@inheritdoc}
   */
  public function cosignCreateNewUser($cosign_name) {
    if (\Drupal::config('cosign.settings')->get('cosign_autocreate') == 1) {
      $new_user = [];
      $new_user['name'] = $cosign_name;
      $new_user['status'] = 1;
      $new_user['password'] = user_password();
      if ($this->cosignIsFriendAccount($cosign_name)) {
        // Friend account.
        $new_user['mail'] = $cosign_name;
      }
      else {
        $new_user['mail'] = $cosign_name . '@' . \Drupal::config('cosign.settings')->get('cosignautocreate_email_domain');
      }
      $account = User::create($new_user);
      $account->enforceIsNew();
      $account->save();

      return $this->cosignLoginUser($account);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function cosignUserMasqueradeCheck() {
    if (isset($_SESSION['masquerading']) && isset($_SESSION['_sf2_attributes']['uid'])) {
      $account = User::load(\Drupal::service('session')->get('uid'));
      $username = $account->getAccountName();
    }
    else {
      $username = $this->cosignRetrieveRemoteUser();
    }
    return $username;
  }

}
